==> sife-0.0.7.tgz: package/lib/componentBuilder.js <==
var fs = require('fs'),
	_ = require('underscore'),
	path = require('path'),
	browserify = require('browserify');


module.exports = {

	init: function(conf) {
		this.conf = conf;
		this.browserify = browserify();
		this.browserify.on('error', function(err){
			console.log('browserify error', err);
		});
	},

	/**
	 * Creates all needed tasks for all components
	 * @param  {gruntHelper} gruntHelper
	 * @param  {string} taskName
	 * @return {void}
	 */
	createTasks: function(gruntHelper, taskName, taskList) {

		var components = this.getComponentsFromFiles(this.conf.sources.components);

		_.each(components, function(component) {

			var files = this.getComponentCssDependencyFiles(component);
			gruntHelper.concat.add(taskName, this.conf.output.css.components + '/' + component.id + '.css', files);

			if(component.dependencies.js){

				// create a single task for each component - for testing/styleguide
				gruntHelper.browserify.add(taskName+'_'+component.id, this.conf.output.js + '/' + component.id + '.js', [path.join(component.path, '/js', component.name+'.js')], {
					browserifyOptions:{
						standalone: 'SIFE.'+component.id.split('/').join('_')
					}
				});
				taskList.push('browserify:'+taskName+'_'+component.id);

			}

		}, this);

	},


	getComponentsFromFiles: function(path) {

		var namespaces = fs.readdirSync(path),
			components = [];

		_.each(namespaces, function(namespace) {

			if (fs.statSync(path + '/' + namespace).isDirectory()) {
				this.addComponentsFromNamespace(namespace, path, components);
			}

		}, this);

		return components;

	},


	addComponentsFromNamespace: function(namespace, path, components) {

		var comps = fs.readdirSync(path + '/' + namespace),
			comp,
			namespaceOptions = this.conf.sources.componentsOptions[namespace] || {};

		// ignore namespsaces that are 'cssOnly' (typically utilities or variables)
		if (namespaceOptions.cssOnly) {
			return;
		}

		_.each(comps, function(componentName) {
			if (namespaceOptions.atomic) {

				var atomicEntity = componentName;
				var atomicEntities = fs.readdirSync(path + '/' + namespace + '/' + atomicEntity);

				_.each(atomicEntities, function(componentName) {

					comp = {
						name: componentName,
						namespace: namespace,
						path: path + '/' + namespace + '/' + atomicEntity + '/' + componentName,
						id: namespace + '/' + atomicEntity + '/' + componentName
					};
					comp.dependencies = this.findComponentDependencies(comp),
					components.push(comp);

				}, this);

			} else {

				comp = {
					name: componentName,
					namespace: namespace,
					path: path + '/' + namespace + '/' + componentName,
					id: namespace + '/' + componentName
				};
				comp.dependencies = this.findComponentDependencies(comp),
				components.push(comp);

			}


		}, this);


	},


	/**
	 * Returns an object containing following keys:
	 * - explicit: array of components described in config.json
	 * - variations: array of variations as found in css/ folder of component
	 * - layout: componentString of the component's layout
	 * - js: boolean
	 * @param  {[type]} componentString [description]
	 * @return {[type]}                 [description]
	 */
	findComponentDependencies: function(component) {

		var config = this.getComponentConfig(component.id);
		var deps = {};

		if (config.dependencies) {
			deps.explicit = config.dependencies;
		}

		var variations = this.getComponentVariations(component.id);

		if (variations) {
			deps.variations = variations;
		}

		if (config.layout) {
			deps.layout = config.layout;
		}

		if (this.getComponentJsModule(component)) {
			deps.js = true;
		}


		return deps;

	},


	getComponentJsModule: function(component){
		return fs.existsSync(path.join(this.conf.sources.components, component.id, 'js', component.name+'.js'));
	},


	/**
	 * Returns the given component's config
	 * (from config.json) or {} if no config.json
	 * @param  {componentString} componentString
	 * @return {object}
	 */
	getComponentConfig: function(componentString) {

		var fileContent;

		try {
			fileContent = fs.readFileSync(this.conf.sources.components + '/' + componentString + '/config.json');
		}

		catch (err) {
			fileContent = '{}';
		}

		return JSON.parse(fileContent);

	},


	/**
	 * Returns an array of all variations for a given componentString
	 * @param  {string} componentString
	 * @return {array} or null if none
	 */
	getComponentVariations: function(componentString) {

		var variations;

		try {
			variations = fs.readdirSync(this.conf.sources.components + '/' + componentString + '/css');
		}

		catch (err) {
			variations = null;
		}

		if (variations) {

			variations = _.map(variations, function(f) {
				return f.replace('.css', '');
			});
			variations = _.filter(variations, function(f) {
				return f.match(/--/);
			});

			if (variations.length === 0) {
				variations = null;
			}

		}

		return variations;

	},

	/**
	 * Returns an array of CSS files the given component
	 * depends on
	 * @param  {object} component Component object
	 * @return {array}
	 */
	getComponentCssDependencyFiles: function(component) {

		// [options.prependFiles]
		// the component css
		// component's dependencies (--variations)
		// excplicit dependencies (in config.json)
		// layout dependencies
		var files = [];

		// for all components, prepend files
		if (this.conf.build.componentsOptions && this.conf.build.componentsOptions.prependFiles) {
			files = files.concat(this.conf.build.componentsOptions.prependFiles);
		}

		// the component's own CSS
		files.push(component.path + '/css/' + component.name + '.css');

		// all variations CSS
		if (component.dependencies.variations) {
			files = files.concat(_.map(component.dependencies.variations, function(variation) {
				return component.path + '/css/' + variation + '.css';
			}));
		}

		// the layout CSS
		if (component.dependencies.layout) {

			var layoutName = component.dependencies.layout.split('/').pop();
			files.push(this.conf.output.css.components + '/' + component.dependencies.layout + '/css/' + layoutName + '.css');

		}

		return files;

	}
};

==> sife-0.0.11.tgz: package/lib/componentBuilder.js <==
var fs = require('fs'),
	_ = require('underscore'),
	path = require('path');


module.exports = {

	init: function(conf) {
		this.conf = conf;
	},

	/**
	 * Creates all needed tasks for all components
	 * @param  {gruntHelper} gruntHelper
	 * @param  {string} taskName
	 * @return {void}
	 */
	createTasks: function(gruntHelper, taskName, taskList) {

		var components = this.getComponentsFromFiles(this.conf.sources.components);

		_.each(components, function(component) {

			var files = this.getComponentCssDependencyFiles(component);
			gruntHelper.concat.add(taskName, this.conf.output.css.components + '/' + component.id + '.css', files);

			if(component.dependencies.js){

				// create a single task for each component - for testing/styleguide
				gruntHelper.browserify.add(taskName+'_'+component.id, this.conf.output.js + '/' + component.id + '.js', [path.join(component.path, '/js', component.name+'.js')], {
					browserifyOptions:{
						standalone: 'SIFE.'+component.id.split('/').join('_')
					}
				});
				taskList.push('browserify:'+taskName+'_'+component.id);

			}

		}, this);

	},


	getComponentsFromFiles: function(path) {

		var namespaces = fs.readdirSync(path),
			components = [];

		_.each(namespaces, function(namespace) {

			if (fs.statSync(path + '/' + namespace).isDirectory()) {
				this.addComponentsFromNamespace(namespace, path, components);
			}

		}, this);

		return components;

	},


	addComponentsFromNamespace: function(namespace, path, components) {

		var comps = fs.readdirSync(path + '/' + namespace),
			comp,
			namespaceOptions = this.conf.sources.componentsOptions[namespace] || {};

		// ignore namespsaces that are 'cssOnly' (typically utilities or variables)
		if (namespaceOptions.cssOnly) {
			return;
		}

		_.each(comps, function(componentName) {
			if (namespaceOptions.atomic) {

				var atomicEntity = componentName;
				var atomicEntities = fs.readdirSync(path + '/' + namespace + '/' + atomicEntity);

				_.each(atomicEntities, function(componentName) {

					components.push(this.getComponentById(namespace + '/' + atomicEntity + '/' + componentName));

				}, this);

			} else {

				components.push(this.getComponentById(namespace + '/' + componentName));

			}


		}, this);


	},

	getComponentById: function(componentId){
		var componentIdParts = componentId.split('/'),
			namespace = componentIdParts.shift(),
			componentName = componentIdParts.pop();


		var comp = {
			name: componentName,
			namespace: namespace,
			path: path.join(this.conf.sources.components ,componentId),
			id: componentId
		};
		comp.dependencies = this.findComponentDependencies(comp);
		return comp;
	},


	/**
	 * Returns an object containing following keys:
	 * - explicit: array of components described in config.json
	 * - variations: array of variations as found in css/ folder of component
	 * - layout: componentString of the component's layout
	 * - js: boolean
	 * @param  {[type]} componentString [description]
	 * @return {[type]}                 [description]
	 */
	findComponentDependencies: function(component) {

		var config = this.getComponentConfig(component.id);
		var deps = {};

		if (config.dependencies) {
			deps.explicit = config.dependencies;
		}

		var variations = this.getComponentVariations(component.id);

		if (variations) {
			deps.variations = variations;
		}

		if (config.layout) {
			deps.layout = config.layout;
		}

		if (this.getComponentJsModule(component)) {
			deps.js = true;
		}


		return deps;

	},


	getComponentJsModule: function(component){
		return fs.existsSync(path.join(this.conf.sources.components, component.id, 'js', component.name+'.js'));
	},


	/**
	 * Returns the given component's config
	 * (from config.json) or {} if no config.json
	 * @param  {componentString} componentString
	 * @return {object}
	 */
	getComponentConfig: function(componentString) {

		var fileContent;

		try {
			fileContent = fs.readFileSync(this.conf.sources.components + '/' + componentString + '/config.json');
		}

		catch (err) {
			fileContent = '{}';
		}

		return JSON.parse(fileContent);

	},


	/**
	 * Returns an array of all variations for a given componentString
	 * @param  {string} componentString
	 * @return {array} or null if none
	 */
	getComponentVariations: function(componentString) {

		var variations;

		try {
			variations = fs.readdirSync(this.conf.sources.components + '/' + componentString + '/css');
		}

		catch (err) {
			variations = null;
		}

		if (variations) {

			variations = _.map(variations, function(f) {
				return f.replace('.css', '');
			});
			variations = _.filter(variations, function(f) {
				return f.match(/--/);
			});

			if (variations.length === 0) {
				variations = null;
			}

		}

		return variations;

	},

	/**
	 * Returns an array of CSS files the given component
	 * depends on
	 * @param  {object} component Component object
	 * @param  {boolean} isDependency
	 * @return {array}
	 */
	getComponentCssDependencyFiles: function(component, isDependency) {

		// [options.prependFiles]
		// the component css
		// component's dependencies (--variations)
		// excplicit dependencies (in config.json)
		// layout dependencies
		var files = [];

		// for all components, prepend files
		if (!isDependency && this.conf.build.componentsOptions && this.conf.build.componentsOptions.prependFiles) {
			files = files.concat(this.conf.build.componentsOptions.prependFiles);
		}

		// the component's own CSS
		files.push(component.path + '/css/' + component.name + '.css');

		// all variations CSS
		if (component.dependencies.variations) {
			files = files.concat(_.map(component.dependencies.variations, function(variation) {
				return component.path + '/css/' + variation + '.css';
			}));
		}

		// the explicit components it depends on
		if (component.dependencies.explicit && component.dependencies.explicit.length > 0) {
			_.each(component.dependencies.explicit, function(dependencyComponentId){
				var dependencyComponent = this.getComponentById(dependencyComponentId);
				files = files.concat(this.getComponentCssDependencyFiles(dependencyComponent, true));
			},this);
		}

		// the layout CSS
		if (component.dependencies.layout) {

			var layoutName = component.dependencies.layout.split('/').pop();
			files.push(this.conf.output.css.components + '/' + component.dependencies.layout + '/css/' + layoutName + '.css');

		}


		return files;

	}
};
